// Copyright 2009 Thomas Hoppe
// -----------------------------------------
// This work is licensed under the Creative Commons Attribution License. 
// To view a copy of this license, visit http://creativecommons.org/licenses/by/2.0/ 
// or send a letter to Creative Commons, 559 Nathan Abbott Way, Stanford, California 94305, USA.
// You are free:
// - to copy, distribute, display, and perform the work
// - to make derivative works
// - to make commercial use of the work
// Under the following conditions:
// - Attribution. You must give the original author credit.
// - For any reuse or distribution, you must make clear to others the license terms of this work.
// - Any of these conditions can be waived if you get permission from the copyright holder.
// Your fair use and other rights are in no way affected by the above. 
                       
// simple std cam                       
#macro Camera(From, To)                 
camera
{
	location  From
	direction 1.5*z
	right     x*image_width/image_height
	look_at   To
}
#end

// include in a union during contruction, creates a .5 raster in x-y plane Len x Len in size
// can be rotated using Rot to show along in other orientations
#macro UnitRaster(LenX, LenY, Rot)
object
{
	box 
	{ 
		<-LenX, -0.01, -LenY>, <LenX, 0.01, LenY>   
		texture
		{
			pigment 
			{ 
				gradient x
				color_map
				{
					[ 0.00 color Cyan transmit 0 ]
					[ 0.02 color Cyan transmit 0 ]
					[ 0.02 color Cyan transmit 1 ]
					[ 0.49 color Cyan transmit 1 ]
					[ 0.49 color Red  transmit 0 ]
					[ 0.51 color Red  transmit 0 ]
					[ 0.51 color Cyan transmit 1 ]
					[ 0.98 color Cyan transmit 1 ]
					[ 0.98 color Cyan transmit 0 ]
					[ 1.00 color Cyan transmit 0 ]
				}
			}                 
			finish
			{
				ambient 1
				diffuse 0
			}
		}
		texture
		{
			pigment 
			{ 
				gradient z
				color_map
				{
					[ 0.00 color Green transmit 0 ]
					[ 0.02 color Green transmit 0 ]
					[ 0.02 color Green transmit 1 ]
					[ 0.49 color Green transmit 1 ]
					[ 0.49 color Magenta  transmit 0 ]
					[ 0.51 color Magenta  transmit 0 ]
					[ 0.51 color Green transmit 1 ]
					[ 0.98 color Green transmit 1 ]
					[ 0.98 color Green transmit 0 ]
					[ 1.00 color Green transmit 0 ]
				}
			}                 
			finish
			{
				ambient 1
				diffuse 0
			}
		}
	}
 	rotate Rot
}
#end         

// set radiosity to nearly std values, some options choosably by params
#macro StdRadiosity(Count, PretraceEnd, PretraceStart, Brightness)
global_settings {
	radiosity {
		pretrace_start PretraceStart  // start pretrace at this size
		pretrace_end   PretraceEnd    // end pretrace at this size
		count Count                   // higher -> higher quality (1..1600) [35]
		nearest_count 5               // higher -> higher quality (1..10) [5]
		error_bound 1.8               // higher -> smoother, less accurate [1.8]
		recursion_limit 3             // how much interreflections are calculated (1..5+) [3]
		low_error_factor .5           // reduce error_bound during last pretrace step
		gray_threshold 0.0            // increase for weakening colors (0..1) [0]
		minimum_reuse 0.015           // reuse of old radiosity samples [0.015]
		brightness Brightness         // brightness of radiosity effects (0..1) [1]
		
		adc_bailout 0.01/2
		//normal on                   // take surface normals into account [off]
		//media on                    // take media into account [off]
		//save_file "file_name"       // save radiosity data
		//load_file "file_name"       // load saved radiosity data
		//always_sample off           // turn sampling in final trace off [on]
		//max_sample 1.0              // maximum brightness of samples
	}
} 
#end      
                      
// common global settings                      
#macro StdGlobal(MaxTraceLevel)
global_settings {
	assumed_gamma 1.0
	max_trace_level MaxTraceLevel
}     
#end                     
    
// as the name says
#macro BlueSkySphere()
sky_sphere {
	pigment {
		gradient y
		color_map {
			[0.0 rgb <0.6,0.7,0.8>]
			[0.7 rgb <0.4,0.5,0.8>]
		}
	}
}
#end    

#macro _BaseArrow(Len, Thick, Color, Rot)
union
{
	cylinder { 0, x*Len, Thick }
	cone { x*Len, 2*Thick, x*(Len+4*Thick), 0 }
	pigment { color Color }
	rotate Rot
}
#end

#macro BaseArrows(Len, Thick, Pos, Rot)
union
{
	_BaseArrow(Len, Thick, Red, 0)
	_BaseArrow(Len, Thick, Green, z*90)
	_BaseArrow(Len, Thick, Blue, y*-90)   
	rotate Rot
	translate Pos
}
#end

// Randomize the given x, y, z values in range -x/2 .. x/2, y and z respective
// using the given seed s.
#macro vrnd(X, Y, Z, S)
	<X*rand(S)-X/2, Y*rand(S)-Y/2, Z*rand(S)-Z/2>
#end                
  
// Search for nearest intersection from point P with Obj, searches angels
// of 360/NA0 or NA1 in all directions. If no Intersection is found, return <0, 0, 0>,
// otherwise, the intersection point is returned.
// In case of an intersection, Norm is set as well.
#macro ClampOld(P, Obj, NA, Norm)                               
	//#debug concat("------ Search ISect: ", vstr(3, P, ", ", 0, 2), "\n")
	#local R=<0, 0, 0>;															// PreDef return value
	#local MDist=1000000000;													// Huge dist for "nearest"-check.
	#local N0=0;																// Outer loop var.
	#while(N0<NA)                                								// Outer loop
		#local Ra=sin((N0/NA)*2*pi);    										// Radius at this height
		#local H=cos((N0/NA)*2*pi);											// Height
	    #local N1=0;															// Inner loop var.
	    #while(N1<NA)															// Inner loop
			#local D1=<Ra*sin((N1/NA)*2*pi), H, Ra*cos((N1/NA)*2*pi)>;		// Direction around y, pointing in up/down based on outer loop
			#local N=<0, 0, 0>;													// Normal for trace call.
			#local Int=trace(Obj, P, D1, N);									// Trace call.
			#if(vlength(N)!=0) 													// Intersect ?     
				#local Dist=vlength(Int-P);										// Distance from point to intersection.
				//#debug concat("  I: ", vstr(3, Int, ", ", 0, 2), " - MDist/Dist:", str(MDist, 0, 2), " - ", str(Dist, 0, 2), "\n")
				#if(Dist<MDist)													// Nearest detected intersection ?    
					//#debug "    hit\n"
					#local R=Int;												// Yes, store intersection.
					#declare Norm=N;											// Store normal from intersection.
					#local MDist=Dist;											// Save distance for next near check.
				#end
			#end
	    	#local N1=N1+1;														// Next inner loop.
	    #end
		#local N0=N0+1;															// Next outer loop.
	#end        
	R																			// Return intersection.
#end
                 
#macro _ISect(P, Dir, Obj, ISect, Norm)
	#local Dist=100000000;
	#local N=<0, 0, 0>;
	#local Int=trace(Obj, P, Dir, N);
	#if(vlength(N)!=0)
		#declare ISect=Int;
		#declare Norm=N;
	    #local Dist=vlength(Int-P);
	#end
	Dist
#end                  

#macro Clamp(P, Obj, Grain, Norm, Debug)                                    
	#if(Debug=true)
		#debug concat("------ Search ISect: ", vstr(3, P, ", ", 0, 2), "\n")
	#end
	#local R=<0, 0, 0>;
	#local MDist=100000000;
	#local Dir=0;
	#while(Dir < 6) // loop over all axis directions
		#switch(Dir)
			#case(0)
				#local SVec=x;
				#local DVec0=y;
				#local DVec1=z;
				#break;
			#case(1)
				#local SVec=-x;
				#local DVec0=y;
				#local DVec1=z;
				#break;
			#case(2)
				#local SVec=y;
				#local DVec0=x;
				#local DVec1=z;
				#break;
			#case(3)
				#local SVec=-y;
				#local DVec0=x;
				#local DVec1=z;
				#break;
			#case(4)
				#local SVec=z;
				#local DVec0=x;
				#local DVec1=y;
				#break;
			#case(5)
				#local SVec=-z;
				#local DVec0=-x;
				#local DVec1=-y;
				#break;
	    #end  
	    
	    //#if(Debug=true)
	    //	#debug concat("Dir = ", str(Dir, 0, 0), "\n")
	    //#end
                     
        // scan every axis in Grain x Grain grid.
		#local L0=-1;
		#while(L0 <= 1)     
		
			#local L1=-1;
			#while(L1 <= 1)
		                                               
		        // calculate vector with current axis/gridpos
				#local D=SVec + DVec0*L0 + DVec1*L1;               
				
				// check if it intersects
				#local N=<0, 0, 0>;        
				#local I=<0, 0, 0>;
				#local Dist=_ISect(P, D, Obj, I, N);
				#if(vlength(N)!=0)                 
					#if(Debug=true)                                                                               
						#debug concat("  D: ", vstr(3, D, ", ", 0, 2), "  I: ", vstr(3, I, ", ", 0, 2), " - MDist/Dist:", str(MDist, 0, 2), " - ", str(Dist, 0, 2), "\n")
					#end
					
					// it does !, check if this is the closest
					#if(Dist<MDist)                                      
						#if(Debug=true)
							#debug "    hit\n"
						#end
						// this is currently the closest, save for return
						#local MDist=Dist;
						#local R=I;
						#declare Norm=N;					
					#end
				#end
		  	    //#debug concat("test dir: ", vstr(3, D, ", ", 0, 2), "\n")
		  		#local L1=L1+2/Grain;
		  	#end
			#local L0=L0+2/Grain;
		#end
		
		#local Dir=Dir+1;   
	#end 
 	R              
#end  
             